# -*- coding: utf-8 -*-
#
# @project TirjaPy
# @file src/tirjapy/utils/RazorpayHandle.py
# @author  Shreos Roychowdhury <shreos@tirja.com>
# @version 1.0.0
# 
# @section DESCRIPTION
# 
#   RazorpayHandle.py : razorpay handle
# 
# @section LICENSE
# 
# Copyright (c) 2025 Shreos Roychowdhury.
# Copyright (c) 2025 Tirja Consulting LLP.
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
# FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
# COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
# IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
# 

from loguru import logger

import os
import sys
import re
import json
import urllib3
from base64 import b64encode

from tirjapy.utils.HttpPostHandle import HttpPostHandle

RAZORPAY_BASE_URL                   = 'https://api.razorpay.com'
RAZORPAY_V1                         = RAZORPAY_BASE_URL + '/v1'
RAZORPAY_V2                         = RAZORPAY_BASE_URL + '/v2'

RAZORPAY_ORDER_URL                  = RAZORPAY_V1 +  '/orders'
RAZORPAY_INVOICE_URL                = RAZORPAY_V1 +  '/invoices'
RAZORPAY_PAYMENT_LINK_URL           = RAZORPAY_V1 +  '/payment_links'
RAZORPAY_PAYMENTS_URL               = RAZORPAY_V1 +  '/payments'
RAZORPAY_REFUNDS_URL                = RAZORPAY_V1 +  '/refunds'
RAZORPAY_CARD_URL                   = RAZORPAY_V1 +  '/cards'
RAZORPAY_CUSTOMER_URL               = RAZORPAY_V1 +  '/customers'
RAZORPAY_TRANSFER_URL               = RAZORPAY_V1 +  '/transfers'
RAZORPAY_VIRTUAL_ACCOUNT_URL        = RAZORPAY_V1 +  '/virtual_accounts'
RAZORPAY_SUBSCRIPTION_URL           = RAZORPAY_V1 +  '/subscriptions'
RAZORPAY_ADDON_URL                  = RAZORPAY_V1 +  '/addons'
RAZORPAY_PLAN_URL                   = RAZORPAY_V1 +  '/plans'
RAZORPAY_SETTLEMENT_URL             = RAZORPAY_V1 +  '/settlements'
RAZORPAY_ITEM_URL                   = RAZORPAY_V1 +  '/items'
RAZORPAY_QRCODE_URL                 = RAZORPAY_V1 +  '/payments/qr_codes'
RAZORPAY_REGISTRATION_LINK_URL      = RAZORPAY_V1 +  '/subscription_registration'
RAZORPAY_FUND_ACCOUNT_URL           = RAZORPAY_V1 +  '/fund_accounts'
RAZORPAY_ACCOUNT                    = RAZORPAY_V2 +  '/accounts'
RAZORPAY_STAKEHOLDER                = RAZORPAY_V2 +  '/stakeholders'
RAZORPAY_PRODUCT                    = RAZORPAY_V2 +  '/products'
RAZORPAY_TNC                        = RAZORPAY_V1 +  '/tnc'
RAZORPAY_TOKEN                      = RAZORPAY_V1 +  '/tokens'
RAZORPAY_IIN                        = RAZORPAY_V1 +  '/iins'
RAZORPAY_WEBHOOK                    = RAZORPAY_V1 +  '/webhooks'
RAZORPAY_DOCUMENT                   = RAZORPAY_V1 +  '/documents'
RAZORPAY_DISPUTE                    = RAZORPAY_V1 +  '/disputes'
RAZORPAY_DEVICE_ACTIVITY_URL        = RAZORPAY_V1 +  '/devices/activity'

RAZORPAY_USER_AGENT = os.environ.get('RAZORPAY_USER_AGENT', 'Razorpay-Python/1.4.2 TirjaPy/1.1.2')

class RazorpayHandle(HttpPostHandle):

	def __init__(self):
		""" init fx inits handles """
		super().__init__()
		self.username = os.environ["RAZORPAY_KEY_ID"]
		self.passwd = os.environ["RAZORPAY_KEY_SECRET"]

	def _PopulateBasicHeaderCreds(self, headers, username, passwd):
		""" function to load username password in headers """
		if len(username)>0 and len(passwd)>0:
			use_creds = ':'.join([username,passwd])
			base64creds = b64encode(use_creds.encode('ascii'))
			headers['Authorization'] = "Basic {}".format(base64creds.decode('ascii'))
		headers['User-Agent'] = RAZORPAY_USER_AGENT

	def _PostJsonData(self, data, url, username, passwd, post='POST'):
		""" post file as json data """

		if len(url)<6:
			raise ValueError("url cannot be empty")
		## handle headers
		headers = {}
		self._PopulateBasicHeaderCreds( headers, username, passwd )
		headers['Content-Type'] = 'application/json'

		output = { 'error' : True , 'status' : 'unknown' }
		r = self.http.request( post, url, headers=headers, body=json.dumps(data))
		# print(r.status)

		if r.status==200:
			output = json.loads( r.data.decode('utf-8') )
		else:
			try:
				logger.warning("RZP Error: {}" , r.data.decode('utf-8'))
			except:
				logger.warning("RZP Error: UNKNOWN")
		return output

	def _GetJsonData(self, url, username, passwd, fields={}):
		""" get file as json data """

		if len(url)<6:
			raise ValueError("url cannot be empty")
		## handle headers
		headers = {}
		self._PopulateBasicHeaderCreds( headers, username, passwd )
		output = { 'error' : True , 'status' : 'unknown' }

		r = self.http.request( 'GET', url, headers=headers, fields=fields)

		if r.status==200:
			output = json.loads( r.data.decode('utf-8') )
		else:
			try:
				logger.warning("RZP Error: {}" , r.data.decode('utf-8'))
			except:
				logger.warning("RZP Error: UNKNOWN")
		return output

	def GetData(self, url, xdata={}):
		""" get items as json data """
		return self._GetJsonData(url, self.username, self.passwd, xdata)

	def PostData(self, url, xdata):
		""" post items as json data """
		return self._PostJsonData(xdata, url, self.username, self.passwd)

	def PatchData(self, url, xtra, xdata):
		""" patch items as json data """
		return self._PostJsonData(xdata, f"{url}/{xtra}", self.username, self.passwd, 'PATCH')




# -*- coding: utf-8 -*-
#
# @project TirjaPy
# @file src/tirjapy/utils/CalyrexHandle.py
# @author  Shreos Roychowdhury <shreos@tirja.com>
# @version 1.0.0
# 
# @section DESCRIPTION
# 
#   CalyrexHandle.py : handle server urls
# 
# @section LICENSE
# 
# Copyright (c) 2025 Shreos Roychowdhury.
# Copyright (c) 2025 Tirja Consulting LLP.
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
# FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
# COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
# IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
# 

import os
import sys
import json
import re
import uuid
from pathlib import Path
from datetime import datetime, timezone
from Crypto.Cipher import AES
import base64
import random
import string

from tirjapy.utils.HandleQuotes import HandleQuotes

class CalyrexHandle(HandleQuotes):
	"""This class needs init, so the constructor is blank"""

	cx_creds = None

	def __init__(self):
		""" constructor default"""
		pass

	def _SanityCheck(self):
		""" check if globals set """
		if not CalyrexHandle.cx_creds:
			raise ValueError("Globals not set for Calyrex")

	def RegisterGlobals(self, params):
		"""Register Global fx"""
		CalyrexHandle.cx_creds = {
			'domain' : self._RequiredField(params, 'domain'),
			'bucket' : self._RequiredField(params, 'bucket'),
			'skey' : self._RequiredField(params, 'skey'),
			'ttls' : self._RequiredField(params, 'ttls')
		}

	def _MakeAccessKey(self, data):
		""" has `ts` , `user` , `uri`, `ipaddr`"""
		usekey=base64.b64decode(CalyrexHandle.cx_creds['skey'])
		## crypto starts
		BS=AES.block_size
		iv = ''.join(random.choices(string.ascii_lowercase + string.digits, k = BS))
		# aes = AES.new(key, AES.MODE_CBC, bytes(iv,'utf8'))
		aes = AES.new(usekey, AES.MODE_CBC, iv.encode('utf8'))
		pad = BS - len(data) % BS
		data = data + pad * chr(pad)
		encrypted = aes.encrypt(data.encode('utf8'))
		return iv+base64.urlsafe_b64encode(encrypted).decode('utf-8')

	def _MakeAccessUri(self, ts, user, fname, ipaddr='127.0.0.1'):
		""" need `ts` , `user` , `uri`, `ipaddr`"""
		new_fname = fname if fname.startswith('/') else str('/' + fname)
		data = '|'.join([ ts, user, new_fname, ipaddr ])
		return new_fname + '?accesskey=' + self._MakeAccessKey( data )

	def _FormatOneTaskEntry(self, user, data):
		""" removes and adds file fields to Task """

		self._SanityCheck()
		del_fields = ['s3_bucket', 'pptx_fpath','json_fpath']
		## will delete the fields anyway
		for fld in del_fields:
			if fld not in data:
				raise ValueError("Missing Field in data")

		ts = str(int(datetime.now(timezone.utc).replace(tzinfo=timezone.utc).timestamp()))
		dom  = 'https://' + CalyrexHandle.cx_creds['domain']

		## add anyway
		data['pptx_url'] = ''
		data['json_url'] = ''

		if CalyrexHandle.cx_creds['bucket'] == data['s3_bucket']:
			pptx_s3_path = data['pptx_fpath']
			if len(pptx_s3_path)>0:
				data['pptx_url'] = dom + self._MakeAccessUri( ts, user, pptx_s3_path)
			json_s3_path = data['json_fpath']
			if len(json_s3_path)>0:
				data['json_url'] = dom + self._MakeAccessUri( ts, user, json_s3_path)

		## delete the fields anyway
		for fld in del_fields:
			if fld in data:
				del data[fld]

		return data

	def GetURL(self, user, s3_bucket, s3_path):
		""" gets url format filestore data """

		self._SanityCheck()

		## add anyway
		data = ''

		if CalyrexHandle.cx_creds['bucket'] != s3_bucket:
			return data
		if len(s3_path)>0:
			ts = str(int(datetime.now(timezone.utc).replace(tzinfo=timezone.utc).timestamp()))
			dom  = 'https://' + CalyrexHandle.cx_creds['domain']
			data = dom + self._MakeAccessUri( ts, user, s3_path)

		return data




# -*- coding: utf-8 -*-
#
# @project TirjaPy
# @file src/tirjapy/utils/HandleQuotes.py
# @author  Shreos Roychowdhury <shreos@tirja.com>
# @version 1.0.0
# 
# @section DESCRIPTION
# 
#   HandleQuotes.py : Quote handler Base Class
# 
# @section LICENSE
# 
# Copyright (c) 2025 Shreos Roychowdhury.
# Copyright (c) 2025 Tirja Consulting LLP.
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
# FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
# COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
# IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
# 

import os
import sys
import re
from pathlib import Path
import html
import json
from datetime import datetime, timezone
from base64 import b64encode, b64decode

class HandleQuotes:

	def __init__(self):
		""" constructor default"""
		pass

	def _NumStr(self, data):
		""" get num str"""
		return data if data.strip() else '0'

	def _SanitHtml(self, data):
		""" html encode fx"""
		return html.escape(str(data), quote=True).replace("\n",'</br>')

	def _SanitQ(self, data):
		""" add quotes fx private"""
		return str(data).replace("'"," ").replace('"',' ')

	def _SanitL(self, data):
		""" add quotes fx private"""
		return str(data).replace('\n','')

	def _SanitJsonQ(self, data):
		""" add single quotes and protect json"""
		return "'" + json.dumps(data).replace("'","\\'") + "'"

	def _SanitTagQ(self, data):
		""" sanit tags to csv fx private"""
		return str(data).replace("'","&#39;").replace("’","&#39;").replace("`","&#39;").replace('"','&#34;').replace("=","&#61;").replace(',','&comma;')
		# return str(data).replace("'","&#39;").replace('"','&#34;').replace("=","&#61;").replace(',','&comma;')

	def _SanitKey(self, data):
		""" convert dash to underscore private"""
		return str(data).strip().replace("'","").replace('"','').replace("-","_").replace(',','_').replace(' ','_').lower()

	def _SanitUser(self, data):
		""" convert dash to underscore private"""
		return str(data).replace("'","").replace('"','').replace(',','_').lower()

	def _SanitExtreme(self, data):
		""" convert non alnum to underscore"""
		return re.sub(r'\W+', '_', str(data)).lower()

	def _AddSingleQ(self, data):
		""" add quotes fx private"""
		return '"' + str(data).strip('"') + '"'

	def _AddQ(self, data):
		""" add quotes fx private"""
		return '"' + str(data).strip('"') + '"'

	def _AddBackQ(self, data):
		""" add back quotes fx private"""
		return '`' + str(data).strip('`') + '`'

	def _AddNoQ(self, data):
		""" add no quotes fx private"""
		return str(data).strip('"')

	def _JoinEachBackQ(self, data):
		""" join array add back quotes to each fx private"""
		return ', '.join( [self._AddBackQ(str(e)) for e in data] ).strip('"')

	def _JoinEachQ(self, data):
		""" join array add quotes to each fx private"""
		return ', '.join( [self._AddQ(str(e)) for e in data] )

	def _JoinNoQ(self, data):
		""" join array fx private"""
		return ', '.join( [str(e) for e in data] ).strip('"')

	def _JoinQ(self, data):
		""" join array add quotes fx private"""
		return '"' + self._JoinNoQ(data) + '"'

	def _JoinUniqueQ(self, data):
		""" join array unique elements add quotes fx private"""
		done=[]
		for e in data:
			e=str(e)
			if not e in done:
				done.append(e)
		return self._JoinQ(done)

	def _ConvertUnderQ(self, data):
		""" convert underscore to space fx private"""
		return data.replace('_',' ')

	def _BoolNoQ(self, data):
		""" add no quotes bool fx private"""
		return 'TRUE' if data else 'FALSE'

	def _GetPath(self, htpath):
		return htpath if htpath.startswith('/') else Path(Path.cwd(),htpath).as_posix()

	def _GetLast(self, word, delim='::'):
		""" get last of"""
		xx = word.rsplit(delim)
		return xx[ len(xx)-1 ] 

	def _GetFirst(self, word, delim='::'):
		""" get first of"""
		return word.rsplit(delim, 1)[0]

	def _SplitWords(self, words, delim=','):
		""" get array of"""
		out = []
		for word in words.rsplit(delim):
			out.append( word.strip() )
		return out

	def _RequiredField(self, data, field):
		""" init handler required and field from data"""
		if field in data and len(str(data[field])) > 0:
			return data[field]
		raise ValueError("Missing Field: " + field)

	def _RequiredObject(self, data, field):
		""" init handler required for object field from data"""
		if field in data and type(data[field]) is dict:
			return data[field]
		raise ValueError("Missing Object Field: " + field)

	def _RequiredArray(self, data, field):
		""" init handler required for array field from data"""
		if field in data and type(data[field]) is list:
			return data[field]
		raise ValueError("Missing Array Field: " + field)

	def _RequiredInteger(self, data, field):
		""" init handler required int field from data"""
		if field in data:
			return int(data[field])
		raise ValueError("Missing Int Field: " + field)

	def _RequiredFloat(self, data, field):
		""" init handler required float field from data"""
		if field in data:
			return float(data[field])
		raise ValueError("Missing Float Field: " + field)

	def _OptionalField(self, data, field , default=''):
		""" init handler optional and field from data"""
		if field in data and len(str(data[field])) > 0:
			return data[field]
		return default

	def _OptionalMappedField(self, data, field , use_map, default=''):
		""" init handler optional and field from data and map"""
		if field in data and len(str(data[field])) > 0:
			if data[field] in use_map.keys():
				return use_map[ data[field] ]
		return default

	def _OptionalInteger(self, data, field , default=0):
		""" init handler optional int field from data"""
		if field in data:
			return int(data[field])
		return default

	def _OptionalFloat(self, data, field , default=0.0):
		""" init handler optional float field from data"""
		if field in data:
			return float(data[field])
		return default

	def _OptionalBool(self, data, field , default=False):
		""" init handler optional bool field from data"""
		if field in data:
			return bool(data[field])
		return default

	def _OptionalBoolInt(self, data, field , default=0):
		""" init handler optional bool field from data"""
		if field in data:
			return 1 if bool(data[field]) else 0
		return default

	def _YesNoBool(self, data, field, default=False):
		""" init handler yes no bool field from data"""
		if field in data:
			if data[field] in ['YES', 'Yes', 'yes', 'Y', 'y']:
				return True
			elif data[field] in ['NO', 'No', 'no', 'N', 'n']:
				return False
		return default

	def _OptionalObject(self, data, field, default={}):
		""" init handler for object field from data"""
		if field in data and type(data[field]) is dict:
			return data[field]
		return default

	def _OptionalArray(self, data, field, default=[]):
		""" init handler for array field from data"""
		if field in data and type(data[field]) is list:
			return data[field]
		return default

	def _CheckDirExists(self, dname):
		""" check and return the dir path """
		if not os.path.isdir(dname):
			raise ValueError("Not a directory: " + dname)
		return dname

	def _CheckFileExists(self, dname, fname):
		""" check and return the file path """
		path = '/'.join([dname,fname])
		if not os.path.isfile(path):
			raise ValueError("Not a file: " + path)
		return path

	def _MicroS3Path(self):
		""" creates s3 path """
		dt = datetime.now(timezone.utc)
		return "{:08d}/{:06d}/{:06d}".format(
			dt.year * 10000 + dt.month  * 100 + dt.day,
			dt.hour * 10000 + dt.minute * 100 + dt.second,
			dt.microsecond)

	def _MicroFileName(self):
		""" creates filename """
		dt = datetime.now(timezone.utc)
		return "{:08d}_{:06d}_{:06d}".format(
			dt.year * 10000 + dt.month  * 100 + dt.day,
			dt.hour * 10000 + dt.minute * 100 + dt.second,
			dt.microsecond)

	def _JsonToBase64(self, data):
		""" convert json to base 64"""
		return b64encode(json.dumps(data).encode('utf-8')).decode('latin-1')

	def _Base64ToJson(self, data):
		""" convert object base 64 to json """
		return json.loads( b64decode(data) )




# -*- coding: utf-8 -*-
#
# @project TirjaPy
# @file src/tirjapy/utils/HttpPostHandle.py
# @author  Shreos Roychowdhury <shreos@tirja.com>
# @version 1.0.0
# 
# @section DESCRIPTION
# 
#   HttpPostHandle.py : HttpPost Handler
# 
# @section LICENSE
# 
# Copyright (c) 2025 Shreos Roychowdhury.
# Copyright (c) 2025 Tirja Consulting LLP.
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
# FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
# COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
# IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
# 

from loguru import logger

import os
import sys
import re
import json
import urllib3
from base64 import b64encode

from tirjapy.utils.HandleQuotes import HandleQuotes
from tirjapy.base.HandleConstants import \
	HTTP_CONNECT_TIMEOUT, HTTP_READ_TIMEOUT, LONG_CONNECT_TIMEOUT, LONG_READ_TIMEOUT

class HttpPostHandle(HandleQuotes):

	my_http_pool_manager = None

	def __init__(self):
		""" constructor default"""
		self._Initialize()
		self.http = HttpPostHandle.my_http_pool_manager

	def _Initialize(self):
		""" Init fx"""
		if HttpPostHandle.my_http_pool_manager:
			return None
		timeout = urllib3.util.Timeout(connect=LONG_CONNECT_TIMEOUT, read=LONG_READ_TIMEOUT)
		pool_manager = urllib3.PoolManager(timeout=timeout)
		HttpPostHandle.my_http_pool_manager = pool_manager

	def _PopulateBasicHeaderCreds(self, headers, username, passwd):
		""" function to load username password in headers """
		if len(username)>0 and len(passwd)>0:
			use_creds = ':'.join([username,passwd])
			base64creds = b64encode(use_creds.encode('ascii'))
			headers['Authorization'] = "Basic {}".format(base64creds.decode('ascii'))

	def PostFile(self, filename, url, username, passwd):
		""" post file as form file """

		if len(url)<6:
			raise ValueError("url icannot be empty")
		## handle headers
		headers = {}
		self._PopulateBasicHeaderCreds( headers, username, passwd )

		output = { 'error' : True , 'status' : 'unknown' }
		with open(filename,'rb') as f:
			file_data = f.read()
			fields={ "file": ( os.path.basename(filename) , file_data) }
			body, content_type = urllib3.filepost.encode_multipart_formdata(fields)
			headers['Content-Type'] = content_type
			
			r = self.http.request( 'POST', url, headers=headers, body=body)
			if r.status==200:
				output = json.loads( r.data.decode('utf-8') )

		return output

	def PostJsonData(self, data, url, username, passwd):
		""" post file as json data """

		if len(url)<6:
			raise ValueError("url cannot be empty")
		## handle headers
		headers = {}
		self._PopulateBasicHeaderCreds( headers, username, passwd )
		headers['Content-Type'] = 'application/json'

		output = { 'error' : True , 'status' : 'unknown' }
		r = self.http.request( 'POST', url, headers=headers, body=json.dumps(data))
		# print(r.status)

		if r.status==200:
			output = json.loads( r.data.decode('utf-8') )
		return output

	def GetJsonData(self, url, username, passwd, fields={}):
		""" get file as json data """

		if len(url)<6:
			raise ValueError("url cannot be empty")
		## handle headers
		headers = {}
		self._PopulateBasicHeaderCreds( headers, username, passwd )
		output = { 'error' : True , 'status' : 'unknown' }

		r = self.http.request( 'GET', url, headers=headers, fields=fields)

		if r.status==200:
			output = json.loads( r.data.decode('utf-8') )
		return output




# -*- coding: utf-8 -*-
#
# @project TirjaPy
# @file src/tirjapy/utils/TypedEnum.py
# @author  Shreos Roychowdhury <shreos@tirja.com>
# @version 1.0.0
# 
# @section DESCRIPTION
# 
#   TypedEnum.py : type preserving enumeration metaclass.
# 
# @section LICENSE
# 
# Copyright (c) 2025 Shreos Roychowdhury.
# Copyright (c) 2025 Tirja Consulting LLP.
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
# FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
# COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
# IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
# 

class TypedEnum(type):
	"""This metaclass creates an enumeration that preserves isinstance(element, type)."""

	def __new__(mcs, cls, bases, classdict):
		"""Discover the enum members by removing all intrinsics and specials."""
		object_attrs = set(dir(type(cls, (object,), {})))
		member_names = set(classdict.keys()) - object_attrs
		member_names = member_names - set(name for name in member_names if name.startswith("_") and name.endswith("_"))
		new_class = None
		base = None
		for attr in member_names:
			value = classdict[attr]
			if new_class is None:
				# base class for all members is the type of the value
				base = type(classdict[attr])
				ext_bases = (*bases, base)
				new_class = super().__new__(mcs, cls, ext_bases, classdict)
				setattr(new_class, "__member_names__", member_names)
			else:
				if not base == type(classdict[attr]):  # noqa
					raise ValueError("Cannot mix types in TypedEnum")
			new_val = new_class.__new__(new_class, value)
			setattr(new_class, attr, new_val)

		for parent in bases:
			new_names = getattr(parent, "__member_names__", set())
			member_names |= new_names
			for attr in new_names:
				value = getattr(parent, attr)
				if not isinstance(value, base):
					raise ValueError("Cannot mix inherited types in TypedEnum: %s from %s" % (attr, parent))
				# convert all inherited values to the new class
				setattr(new_class, attr, new_class(value))

		return new_class

	def __call__(cls, arg):
		for name in cls.__member_names__:
			if arg == getattr(cls, name):
				return type.__call__(cls, arg)
		raise ValueError("Invalid value '%s' for %s" % (arg, cls.__name__))

	@property
	def __members__(cls):
		"""Sufficient to make the @unique decorator work."""

		class FakeEnum:  # pylint: disable=too-few-public-methods
			"""Object that looks a bit like an Enum instance."""

			def __init__(self, name, value):
				self.name = name
				self.value = value

		return {name: FakeEnum(name, getattr(cls, name)) for name in cls.__member_names__}

	def __iter__(cls):
		"""List all enum values."""
		return (getattr(cls, name) for name in cls.__member_names__)

	def __len__(cls):
		"""Get number of enum values."""
		return len(cls.__member_names__)




# -*- coding: utf-8 -*-
#
# @project TirjaPy
# @file src/tirjapy/base/HandleConstants.py
# @author  Shreos Roychowdhury <shreos@tirja.com>
# @version 1.0.0
# 
# @section DESCRIPTION
# 
#   HandleConstants.py : Handle different timeouts and constants
# 
# @section LICENSE
# 
# Copyright (c) 2025 Shreos Roychowdhury.
# Copyright (c) 2025 Tirja Consulting LLP.
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
# FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
# COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
# IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
# 

COMMA = ','
SPACE = ' '
EQUALTO = '='
NOTEQUALTO = '!='
BLANK = ''
COLON = ':'

# http post

HTTP_CONNECT_TIMEOUT     =     60                 # default http connect timeout
HTTP_READ_TIMEOUT        =    300                 # default http read timeout

LONG_CONNECT_TIMEOUT     =    600                 # long http connect timeout
LONG_READ_TIMEOUT        =    600                 # long http read timeout

# redis

TASK_CANCEL_HSET         = 'nb_def_cancel'        # task cancel hset
TASK_DO_CANCEL           = b'cancel'              # task cancel val




# -*- coding: utf-8 -*-
#
# @project TirjaPy
# @file src/tirjapy/utils/MysqlHandleBase.py
# @author  Shreos Roychowdhury <shreos@tirja.com>
# @version 1.0.0
# 
# @section DESCRIPTION
# 
#   MysqlHandleBase.py :  Mysql handle Base Fx
# 
# @section LICENSE
# 
# Copyright (c) 2025 Shreos Roychowdhury.
# Copyright (c) 2025 Tirja Consulting LLP.
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
# FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
# COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
# IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
# 

from loguru import logger

import os
import sys
import json

import mysql.connector
from mysql.connector import errorcode

from tirjapy.utils.HandleQuotes import HandleQuotes

class MysqlHandleBase(HandleQuotes):
	""" mysql handled class """

	my_creds = None

	def __init__(self):
		""" constructor default"""
		self.is_init = False
		if MysqlHandleBase.my_creds:
			self._InitMysql()

	def __del__(self):
		""" destructor default"""
		if self.is_init:
			## handle "ImportError: No localization support for language 'eng'"
			try:
				self.session.close()
			except:
				pass

	def _InitMysql(self):
		if self.is_init:
			return
		self.session = mysql.connector.connect(
    	host= MysqlHandleBase.my_creds['host'], port= MysqlHandleBase.my_creds['port'],
			user= MysqlHandleBase.my_creds['user'], password= MysqlHandleBase.my_creds['pass'],
			autocommit=True)
		self.db_main = MysqlHandleBase.my_creds['db_main']
		self.is_init = True

	def RegisterGlobals(self, params):
		"""Register Global fx"""

		MysqlHandleBase.my_creds = {
			'host' : self._RequiredField(params, 'host'),
			'port' : self._RequiredField(params, 'port'),
			'user' : self._RequiredField(params, 'user'),
			'pass' : self._RequiredField(params, 'pass'),
			'db_main' : self._RequiredField(params, 'db_main'),
		}
		self._InitMysql()

	def _CheckInit(self):
		if not self.is_init:
			raise ValueError("Mysql Vars not found")

	def _MysqlUpdate(self, db, sql, multi=False):
		""" Internal Mysql Update Handler"""
		session = self.session
		session.database = db
		cursor = session.cursor()

		noerr = False

		try:
			#cursor.execute(sql,multi=multi) ## depre in 9
			cursor.execute(sql)
			noerr = True

		except mysql.connector.Error as err:
			if err.errno == errorcode.ER_TABLE_EXISTS_ERROR:
				logger.warning("table already exists.")
			else:
				logger.warning(err.msg)

		cursor.close()
		return noerr

	def _MysqlUpdateTuple(self, db, sql, tvals):
		""" Internal Mysql Update Handler"""
		session = self.session
		session.database = db
		cursor = session.cursor()

		noerr = False
		try:
			cursor.execute(sql,tvals)
			noerr = True
		except mysql.connector.Error as err:
			logger.warning(err.msg)

		cursor.close()
		return noerr

	def _MysqlSelect(self, db, sql):
		""" Internal Mysql Select Handler"""

		session = self.session
		session.database = db
		cursor = session.cursor()
		data = []
		try:
			cursor.execute(sql)
			while True:
				rows = cursor.fetchmany(1000)
				if not rows:
					break
				data.extend(rows)
		except mysql.connector.Error as err:
			logger.warning(err.msg)

		cursor.close()
		return data

	def _MysqlSelectToWriter(self, db, sql, tsv_writer):
		""" Internal Mysql Select Handler to tsv_writer"""

		session = self.session
		session.database = db
		cursor = session.cursor()
		try:
			cursor.execute(sql)
			while True:
				rows = cursor.fetchmany(1000)
				if not rows:
					break
				tsv_writer.writerows(rows)
		except mysql.connector.Error as err:
			logger.warning(err.msg)

		cursor.close()

	def _MysqlGetLastInsert(self, db, sql):
		""" Mysql Fx to get last id """

		toret = 0
		session = self.session
		session.database = db
		cursor = session.cursor()
		try:
			cursor.execute(sql)
			toret = int(cursor.lastrowid)
		except mysql.connector.Error as err:
			logger.warning(err.msg)

		cursor.close()
		return toret
